﻿#include <iostream>
#include <vector>
#include <cmath>
#include<iterator>
#include<string>
using namespace std;

// 定义页面结构
struct Page {
    double rank;
    vector<int> links;
}typedef Page;

// 计算PageRank
static void calculatePageRank(vector<Page>& pages, double dampingFactor, double epsilon, int maxIterations) {
    int numPages = pages.size();
    vector<double> newRanks(numPages, 1.0 / numPages);//给每个页面权重赋初值

    for (int iteration = 0; iteration < maxIterations; ++iteration) {
        vector<double> prevRanks = newRanks;

        // 遍历每个页面
        for (int i = 0; i < numPages; ++i) {
            double sum = 0.0;

            // 遍历指向当前页面的页面
            for (int j : pages[i].links) {
                sum += prevRanks[j] / pages[j].links.size();
            }

            newRanks[i] = (1.0 - dampingFactor) / numPages + dampingFactor * sum;
        }

        // 检查收敛
        double delta = 0.0;
        for (int i = 0; i < numPages; ++i) {
            delta += abs(newRanks[i] - prevRanks[i]);
        }

        if (delta < epsilon) {
            break;  // 收敛则退出循环
        }
    }

    // 更新页面的PageRank值
    for (int i = 0; i < numPages; ++i) {
        pages[i].rank = newRanks[i];
    }
}



int main() {
    // 输入有向图的二维数组
    //vector<vector<int>> directedGraph = {
    //    {0, 1, 1, 1},  // 页面0链接到页面1
    //    {1, 0, 0, 1},  // 页面1链接到页面2
    //    {1, 0, 0, 0},
    //    {0, 1, 1, 0}
    //};
    vector<vector<int>> directedGraph;

    // 构建页面结构
    int page_count;
    

    cout << "输入页面的数量：" << endl;
    cin >> page_count;
    cout << "输入页面指向矩阵" << endl;

    istream_iterator<int> start(cin),eof;

    for (int i = 0; i < page_count; i++)
    {
        vector<int> a;
        while (start != eof) {
            a.push_back(*start);
            ++start;
            if (cin.peek() == '\n')
            {
                a.push_back(*start);
                break;
            }
        }
        directedGraph.push_back(a);
    }


    vector<Page> pages(page_count);

    for (int i = 0; i < page_count; ++i) {
        for (int j = 0; j < page_count; ++j) {
            if (directedGraph[i][j] == 1) {
                pages[i].links.push_back(j);
            }
        }
    }

    // 设置PageRank算法参数
    double dampingFactor = 0.85;
    double epsilon = 0.0001;
    int maxIterations = 100;

    // 计算PageRank
    calculatePageRank(pages, dampingFactor, epsilon, maxIterations);

    // 输出每个页面的PageRank值
    for (int i = 0; i < page_count; ++i) {
        cout << "Page " << i << ": " << pages[i].rank << endl;
    }

    return 0;
}
/*
4
0 1 1 1
1 0 0 1
1 0 0 0
0 1 1 0
*/